<?php

namespace Laraauto\Automobile\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Laraauto\Automobile\Http\Requests\CarModelRequest;
use Laraauto\Automobile\Interfaces\CarModelRepositoryInterface;
use Laraauto\Automobile\Models\CarModel;

/**
 * Resource controller class for car_model.
 */
class CarModelResourceController extends BaseController
{

    /**
     * Initialize car_model resource controller.
     *
     * @param type CarModelRepositoryInterface $car_model
     *
     * @return null
     */
    public function __construct(CarModelRepositoryInterface $car_model)
    {
        parent::__construct();
        $this->repository = $car_model;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Laraauto\Automobile\Repositories\Criteria\CarModelResourceCriteria::class);
    }

    /**
     * Display a list of car_model.
     *
     * @return Response
     */
    public function index(CarModelRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Laraauto\Automobile\Repositories\Presenter\CarModelPresenter::class)
                ->$function();
        }

        $car_models = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('automobile::car_model.names'))
            ->view('automobile::car_model.index', true)
            ->data(compact('car_models', 'view'))
            ->output();
    }

    /**
     * Display car_model.
     *
     * @param Request $request
     * @param Model   $car_model
     *
     * @return Response
     */
    public function show(CarModelRequest $request, CarModel $car_model)
    {

        if ($car_model->exists) {
            $view = 'automobile::car_model.show';
        } else {
            $view = 'automobile::car_model.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('automobile::car_model.name'))
            ->data(compact('car_model'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new car_model.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CarModelRequest $request)
    {

        $car_model = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('automobile::car_model.name')) 
            ->view('automobile::car_model.create', true) 
            ->data(compact('car_model'))
            ->output();
    }

    /**
     * Create new car_model.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CarModelRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $car_model                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('automobile::car_model.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('automobile/car_model/' . $car_model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/automobile/car_model'))
                ->redirect();
        }

    }

    /**
     * Show car_model for editing.
     *
     * @param Request $request
     * @param Model   $car_model
     *
     * @return Response
     */
    public function edit(CarModelRequest $request, CarModel $car_model)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('automobile::car_model.name'))
            ->view('automobile::car_model.edit', true)
            ->data(compact('car_model'))
            ->output();
    }

    /**
     * Update the car_model.
     *
     * @param Request $request
     * @param Model   $car_model
     *
     * @return Response
     */
    public function update(CarModelRequest $request, CarModel $car_model)
    {
        try {
            $attributes = $request->all();

            $car_model->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('automobile::car_model.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('automobile/car_model/' . $car_model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('automobile/car_model/' . $car_model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the car_model.
     *
     * @param Model   $car_model
     *
     * @return Response
     */
    public function destroy(CarModelRequest $request, CarModel $car_model)
    {
        try {

            $car_model->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('automobile::car_model.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('automobile/car_model/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('automobile/car_model/' . $car_model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple car_model.
     *
     * @param Model   $car_model
     *
     * @return Response
     */
    public function delete(CarModelRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('automobile::car_model.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('automobile/car_model'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/automobile/car_model'))
                ->redirect();
        }

    }

    /**
     * Restore deleted car_models.
     *
     * @param Model   $car_model
     *
     * @return Response
     */
    public function restore(CarModelRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('automobile::car_model.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/automobile/car_model'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/automobile/car_model/'))
                ->redirect();
        }

    }

}
